package cn.ibizlab.api.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;
import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;
import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PostAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import cn.ibizlab.api.dto.*;
import cn.ibizlab.api.mapping.*;
import cn.ibizlab.core.task.domain.JOBLog;
import cn.ibizlab.core.task.service.IJOBLogService;
import cn.ibizlab.core.task.filter.JOBLogSearchContext;

@Slf4j
@Api(tags = {"任务调度日志" })
@RestController("api-joblog")
@RequestMapping("")
public class JOBLogResource {

    @Autowired
    public IJOBLogService joblogService;

    @Autowired
    @Lazy
    public JOBLogMapping joblogMapping;

    @ApiOperation(value = "获取任务调度日志草稿", tags = {"任务调度日志" },  notes = "获取任务调度日志草稿")
	@RequestMapping(method = RequestMethod.GET, value = "/joblogs/getdraft")
    public ResponseEntity<JOBLogDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(joblogMapping.toDto(joblogService.getDraft(new JOBLog())));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLog-Update-all')")
    @ApiOperation(value = "更新任务调度日志", tags = {"任务调度日志" },  notes = "更新任务调度日志")
	@RequestMapping(method = RequestMethod.PUT, value = "/joblogs/{joblog_id}")
    @Transactional
    public ResponseEntity<JOBLogDTO> update(@PathVariable("joblog_id") BigInteger joblog_id, @RequestBody JOBLogDTO joblogdto) {
		JOBLog domain  = joblogMapping.toDomain(joblogdto);
        domain .setId(joblog_id);
		joblogService.update(domain );
		JOBLogDTO dto = joblogMapping.toDto(domain );
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLog-Update-all')")
    @ApiOperation(value = "批量更新任务调度日志", tags = {"任务调度日志" },  notes = "批量更新任务调度日志")
	@RequestMapping(method = RequestMethod.PUT, value = "/joblogs/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<JOBLogDTO> joblogdtos) {
        joblogService.updateBatch(joblogMapping.toDomain(joblogdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLog-Create-all')")
    @ApiOperation(value = "新建任务调度日志", tags = {"任务调度日志" },  notes = "新建任务调度日志")
	@RequestMapping(method = RequestMethod.POST, value = "/joblogs")
    @Transactional
    public ResponseEntity<JOBLogDTO> create(@RequestBody JOBLogDTO joblogdto) {
        JOBLog domain = joblogMapping.toDomain(joblogdto);
		joblogService.create(domain);
        JOBLogDTO dto = joblogMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLog-Create-all')")
    @ApiOperation(value = "批量新建任务调度日志", tags = {"任务调度日志" },  notes = "批量新建任务调度日志")
	@RequestMapping(method = RequestMethod.POST, value = "/joblogs/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<JOBLogDTO> joblogdtos) {
        joblogService.createBatch(joblogMapping.toDomain(joblogdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "检查任务调度日志", tags = {"任务调度日志" },  notes = "检查任务调度日志")
	@RequestMapping(method = RequestMethod.POST, value = "/joblogs/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody JOBLogDTO joblogdto) {
        return  ResponseEntity.status(HttpStatus.OK).body(joblogService.checkKey(joblogMapping.toDomain(joblogdto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLog-Save-all')")
    @ApiOperation(value = "保存任务调度日志", tags = {"任务调度日志" },  notes = "保存任务调度日志")
	@RequestMapping(method = RequestMethod.POST, value = "/joblogs/save")
    public ResponseEntity<Boolean> save(@RequestBody JOBLogDTO joblogdto) {
        return ResponseEntity.status(HttpStatus.OK).body(joblogService.save(joblogMapping.toDomain(joblogdto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLog-Save-all')")
    @ApiOperation(value = "批量保存任务调度日志", tags = {"任务调度日志" },  notes = "批量保存任务调度日志")
	@RequestMapping(method = RequestMethod.POST, value = "/joblogs/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<JOBLogDTO> joblogdtos) {
        joblogService.saveBatch(joblogMapping.toDomain(joblogdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLog-Remove-all')")
    @ApiOperation(value = "删除任务调度日志", tags = {"任务调度日志" },  notes = "删除任务调度日志")
	@RequestMapping(method = RequestMethod.DELETE, value = "/joblogs/{joblog_id}")
    @Transactional
    public ResponseEntity<Boolean> remove(@PathVariable("joblog_id") BigInteger joblog_id) {
         return ResponseEntity.status(HttpStatus.OK).body(joblogService.remove(joblog_id));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLog-Remove-all')")
    @ApiOperation(value = "批量删除任务调度日志", tags = {"任务调度日志" },  notes = "批量删除任务调度日志")
	@RequestMapping(method = RequestMethod.DELETE, value = "/joblogs/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<BigInteger> ids) {
        joblogService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLog-Get-all')")
    @ApiOperation(value = "获取任务调度日志", tags = {"任务调度日志" },  notes = "获取任务调度日志")
	@RequestMapping(method = RequestMethod.GET, value = "/joblogs/{joblog_id}")
    public ResponseEntity<JOBLogDTO> get(@PathVariable("joblog_id") BigInteger joblog_id) {
        JOBLog domain = joblogService.get(joblog_id);
        JOBLogDTO dto = joblogMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLog-Default-all')")
	@ApiOperation(value = "获取DEFAULT", tags = {"任务调度日志" } ,notes = "获取DEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/joblogs/fetchdefault")
	public ResponseEntity<List<JOBLogDTO>> fetchDefault(JOBLogSearchContext context) {
        Page<JOBLog> domains = joblogService.searchDefault(context) ;
        List<JOBLogDTO> list = joblogMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLog-Default-all')")
	@ApiOperation(value = "查询DEFAULT", tags = {"任务调度日志" } ,notes = "查询DEFAULT")
    @RequestMapping(method= RequestMethod.POST , value="/joblogs/searchdefault")
	public ResponseEntity<Page<JOBLogDTO>> searchDefault(@RequestBody JOBLogSearchContext context) {
        Page<JOBLog> domains = joblogService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(joblogMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}
}

